/*
   SPDX-FileCopyrightText: 2013-2022 Laurent Montel <montel@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#pragma once

#include <QString>
namespace FolderArchive
{
namespace FolderArchiveUtil
{
QString groupConfigPattern();
QString configFileName();
}
}
