/*
    SPDX-FileCopyrightText: 2015-2016 Krzysztof Nowicki <krissn@op.pl>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "ewsclient_debug.h"

QTemporaryDir ewsLogDir(QStringLiteral("/tmp/akonadi-ews-XXXXXXX"));
