/*
    SPDX-FileCopyrightText: 2018 Daniel Vrátil <dvratil@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "maildirconfig.h"

AKONADI_AGENTCONFIG_FACTORY(AkonotesConfigFactory, "akonotesconfig.json", MaildirConfig)
#include "akonotesconfig.moc"
